extern crate uinput;
extern crate evdev_rs;
extern crate crossbeam;

use std::thread;
use std::env::args;
use std::time::Duration;

use evdev_rs::enums::EventType::{EV_KEY, EV_REL};

use crossbeam::crossbeam_channel:: Select;

mod virtual_mouse;
mod virtual_kbd;
mod hw_device;
mod event;


fn main() {
    let args: Vec<_> = args().collect();

    // (path, receiver, stopper)
    let mut devices = Vec::new();
    // let mut disabled_devices = Vec::new();

    for device_path in &args[1..] {
        devices.push(hw_device::HwDevice::new(&device_path));
    }

    // // testing device removal
    // thread::sleep(Duration::from_secs(1));
    // devices[1].stop();
    // disabled_devices.push(devices.remove(1));
    // thread::sleep(Duration::from_secs(1));

    // // add it again
    // devices.push(hw_device::HwDevice::new(&args[1]));

    // init virtual devices
    let mut virt_kbd = virtual_kbd::VirtualKeyboard::new("test-kbd");
    let mut virt_mouse = virtual_mouse::VirtualMouse::new("test-mouse");

    // read events and feed them to the virtual devices
    loop {
        // init select
        let mut sel = Select::new();

        // add select operations
        for d in &devices {
            sel.recv(&d.event_channel);
        }

        // select from available device(s)
        let oper = sel.select();

        // read from selected device
        let index = oper.index();
        match oper.recv(&devices[index].event_channel) {
            Ok(e) => {
                // mouse event
                if e.typ == EV_REL as u32 || (e.code >= 272 && e.code <= 276) {
                    virt_mouse.device.write(e.typ as i32, e.code as i32, e.value).unwrap();
                    virt_mouse.device.synchronize().unwrap();
                // keyboard event
                } else if e.typ == EV_KEY as u32 {
                    virt_kbd.device.write(e.typ as i32, e.code as i32, e.value).unwrap();
                    virt_kbd.device.synchronize().unwrap();
                }
            },
            _ => (),
        }
    }
}
